@@ -4,13 +4,17 @@
 # related or neighboring rights to the content of this file.
 # Attribution is requested but is not required.
 
-# t_TestCommon -- setup and tools specifically for tests. 
+# t_TestCommon -- setup and tools specifically for tests.
 #
 #   Everything not specifically test-related is in the separate
 #   module t_Common (which is not necessairly just for tests).
 #
-#   Loads Test2::V0, which sets UTF-8 encoding/decoding for all file handles
-#   (and imports 'utf8').
+#   Loads Test2::V0 (except with ":no-Test2"), which sets UTF-8 enc/dec
+#   for test-harnes streams (but *not* STD* or new filehandles).
+#
+#   Imports 'utf8' so scripts can be written in UTF-8 encoding
+#
+#   Makes STDIN, STDOUT & STDERR UTF-8 auto de/encode
 #
 #   warnings are *not* imported, to avoid clobbering 'no warnings ...'
 #   settings done beforehand (e.g. via t_Common).
@@ -39,28 +43,38 @@ BEGIN{
   confess "Test::More already loaded!" if defined( &Test::More::ok );
   confess "Test2::V0 already loaded!" if defined( &Test2::V0::import );
 
-  # Now redundant with Test2::V0
-  ## Maybe we should just call binmode(encoding...) on STDOUT & STDERR?
-  #use open IO => ':encoding(UTF-8)', ':std';
+  # Force UTF-8 (and remove any other encoder) regardless of the 
+  # environment/terminal.  This allows tests to use capture {...} and check 
+  # the results independent of the environment, even if when printed the 
+  # results may be garbled.
+  binmode(STDIN, ":raw:encoding(UTF-8):crlf");
+  if ($^O eq "MSWin32") {
+    binmode(STDOUT, ":raw:encoding(UTF-8)");
+    binmode(STDERR, ":raw:encoding(UTF-8)");
+  } else {
+    binmode(STDOUT, ":raw:crlf:encoding(UTF-8)");
+    binmode(STDERR, ":raw:crlf:encoding(UTF-8)");
+  }
 
   # Disable buffering
   STDERR->autoflush(1);
   STDOUT->autoflush(1);
 }
-use Test2::V0 (); # a huge collection of tools
 use POSIX ();
+use utf8;
+use JSON ();
 
 require Exporter;
 use parent 'Exporter';
 our @EXPORT = qw/silent
-                 bug 
+                 bug
                  t_ok t_is t_like
                  ok_with_lineno is_with_lineno like_with_lineno
-                 rawstr showstr showcontrols displaystr 
+                 rawstr showstr showcontrols displaystr
                  show_white show_empty_string
-                 fmt_codestring 
-                 verif_no_internals_mentioned 
-                 insert_loc_in_evalstr verif_eval_err 
+                 fmt_codestring
+                 verif_no_internals_mentioned
+                 insert_loc_in_evalstr verif_eval_err
                  timed_run
                  mycheckeq_literal expect1 mycheck _mycheck_end
                  arrays_eq hash_subset
@@ -75,10 +89,11 @@ use Data::Dumper;
 use Cwd qw/getcwd abs_path/;
 use POSIX qw/INT_MAX/;
 use File::Basename qw/dirname/;
+use Env qw/@PATH @PERL5LIB/;  # ties @PATH, @PERL5LIB
 
 sub bug(@) { @_=("BUG FOUND:",@_); goto &Carp::confess }
 
-# Parse manual-testing args from @ARGV 
+# Parse manual-testing args from @ARGV
 my @orig_ARGV = @ARGV;
 our ($debug, $verbose, $silent, $nonrandom);
 use Getopt::Long qw(GetOptions);
@@ -107,7 +122,7 @@ if ($nonrandom) {
     $ENV{PERL_PERTURB_KEYS} = "2"; # deterministic
     $ENV{PERL_HASH_SEED} = "0xDEADBEEF";
     #$ENV{PERL_HASH_SEED_DEBUG} = "1";
-    $ENV{PERL5LIB} = join(":", @INC);
+    @PERL5LIB = @INC; # cf 'use Env' above
     exec $^X, $0, @orig_ARGV; # for reproducible results
   }
 }
@@ -115,25 +130,43 @@ if ($nonrandom) {
 sub import {
   my $target = caller;
 
+  my %tags;
+  for (my $ix=0; $ix <= $#_; $ix++) {
+    if ($_[$ix] =~ /^(:.*)$/) {
+      next if $_[$ix] eq ":DEFAULT"; # ok, pass thru to Exporter
+      $tags{$1} = 1;
+      splice @_, $ix, 1, ();
+      redo unless $ix > $#_;
+    }
+  }
+
   # Do an initial read of $[ so arybase will be autoloaded
   # (prevents corrupting $!/ERRNO in subsequent tests)
   eval '$[' // die;
 
   # Test2::V0
   #  Do not import warnings, to avoid un-doing prior settings.
-  #  Do not inport 1- and 2- or 3- character upper-case names, which are 
+  #  Do not inport 1- and 2- or 3- character upper-case names, which are
   #  likely to clash with user variables and/or spreadsheet column letters
   #  (when using Spreadsheet::Edit).
-  Test2::V0->import::into($target,
-    -no_warnings => 1,
-    (map{ "!$_" } "A".."AAZ")
-  );
+  unless (delete $tags{":no-Test2"}) {
+    require Test2::V0; # a huge collection of tools
+    require Test2::Plugin::BailOnFail;
+    Test2::V0->import::into($target,
+      -no_warnings => 1,
+      (map{ "!$_" } "A".."AAZ")
+    );
+    # Stop on the first error
+    Test2::Plugin::BailOnFail->import::into($target);
+  }
+  utf8->import::into($target);
 
-  if (grep{ $_ eq ':silent' } @_) {
-    @_ = grep{ $_ ne ':silent' } @_;
+  if (delete $tags{":silent"}) {
     _start_silent() unless $debug;
   }
 
+  die "Unhandled tag ",keys(%tags) if keys(%tags);
+
   # chain to Exporter to export any other importable items
   goto &Exporter::import
 }
@@ -157,29 +190,71 @@ sub hash_subset($@) {
   return { map { exists($hash->{$_}) ? ($_ => $hash->{$_}) : () } @keys }
 }
 
-# string_to_tempfile($string, args => for-mytempfile) 
+# string_to_tempfile($string, args => for-mytempfile)
 # string_to_tempfile($string, pseudo_template) # see mytempfile
 #
 sub string_to_tempfile($@) {
   my ($string, @tfargs) = @_;
   my ($fh, $path) = mytempfile(@tfargs);
   dprint "> Creating $path\n";
-  print $fh $string; 
+  print $fh $string;
   $fh->flush;
   seek($fh,0,0) or die "seek $path : $!";
   wantarray ? ($path,$fh) : $path
 }
 
-# Run a Perl script in a sub-process.
-# Plain 'system  path/to/script.pl' does not work in a test environment
-# where the correct Perl executable is not at the front of PATH,
-# and also where -I options might supply library paths.
-# This is usually enclosed in Capture { ... }
+# Run a Perl script in a sub-process, forcing UTF-8 encoding.
+#
+# -CIOE is passed and LANG is unconditionally set to indicate UTF-8 stdio 
+# regardless of the actual test environment.  This is because the output 
+# will be consumed by a test checker rather than actually displayed 
+# on the real STDOUT/ERR (except for debugging, when results may be garbled).
+#
+# Also provides -I options to mimic @INC (PERL5LIB is often not set)
+#
+# This is usually enclosed in Tiny::Capture::capture { ... }
+#    ==> IMPORTANT: Be sure STDOUT/ERR has :encoding(...) set beforehand
+#        because Tiny::Capture will decode captured output the same way.
+#        Otherwise wide chars will be corrupted
+#
 sub run_perlscript(@) {
-  my @perlargs = @_;  # might be ('-e', 'perlcode...')
+  my $tf; # keep in scope until no longer needed
+  my @perlargs = ("-CIOE", @_);
+  @perlargs = ((map{ "-I$_" } @INC), @perlargs);
   unshift @perlargs, "-MCarp=verbose" if $Carp::Verbose;
-  local $ENV{PERL5LIB} = join(":", @INC);
-  system $^X, @perlargs;
+  unshift @perlargs, "-MCarp::Always=verbose" if $Carp::Always::Verbose;
+  if ($^O eq "MSWin32") {
+    for (my $ix=0; $ix <= $#perlargs; $ix++) {
+      if ($perlargs[$ix] =~ /^-w?[Ee]$/) {
+        # Passing perl code in an argument is impractical in DOS/Windows
+        $tf = Path::Tiny->tempfile("perlcode_XXXXX");
+        $tf->spew_utf8($perlargs[$ix+1]);
+        splice(@perlargs, $ix, 2, $tf->stringify);
+      }
+      for ($perlargs[$ix]) {
+        if (/^-\*[Ee]/) { oops "unhandled perl arg" }
+        s/"/\\"/g;
+        if (/[\s\/"']/) {
+          $_ = '"' . $_ . '"';
+        }
+      }
+    }
+  }
+  delete local $ENV{LC_ALL};
+  local $ENV{LANG} = "C.UTF-8";
+  if ($debug) {
+    my $msg = "%%% run_perlscript >";
+    for my $k (sort keys %ENV) { 
+      next unless $k =~ /^(LC|LANG)/;
+      $msg .= " $k='$ENV{$k}'" 
+    }
+    $msg .= " $^X";
+    $msg .= " '${_}'" foreach (@perlargs);
+    print STDERR "$msg\n";
+  }
+  my $wstat = system $^X, @perlargs;
+  print STDERR "%%%(returned from 'system', wstat=",sprintf("0x%04X",$wstat),")%%%\n" if $debug;
+  $wstat
 }
 
 #--------------- :silent support ---------------------------
@@ -225,12 +300,12 @@ sub _start_silent() {
   $silent_mode = 1;
 
   $orig_DIE_trap = $SIG{__DIE__};
-  $SIG{__DIE__} = sub{ 
+  $SIG{__DIE__} = sub{
     return if $^S or !defined($^S);  # executing an eval, or Perl compiler
-    my @diemsg = @_; 
+    my @diemsg = @_;
     my $err=_finish_silent(); warn $err if $err;
     die @diemsg;
-  }; 
+  };
 
   my @OUT_layers = grep{ $_ ne "unix" } PerlIO::get_layers(*STDOUT, output=>1);
   open($orig_stdOUT, ">&", \*STDOUT) or die "dup STDOUT: $!";
@@ -272,32 +347,45 @@ END{
 }
 #--------------- (end of :silent stuff) ---------------------------
 
-# N.B. package dir might have version like ".../ODF-lpOD_Helper-3.008/..."
-dirname(abs_path(__FILE__)) =~ m#.*/(\w+)-\w[-\w\.]*/# or die "Cant intuit testee module name";
-(my $testee_top_module = $1) =~ s/-/::/g;
+# Find the ancestor build or checkout directory (it contains a "lib" subdir)
+# and derive the package name from e.g. "My-Pack" or "My-Pack-1.234"
+my $testee_top_module;
+for (my $path=path(__FILE__); 
+             $path ne Path::Tiny->rootdir; $path=$path->parent) {
+  if (-e (my $p = $path->child("dist.ini"))) {
+    $p->slurp() =~ /^ *name *= *(\S+)/i or oops;
+    ($testee_top_module = $1) =~ s/-/::/g;
+    last
+  }
+  if (-e (my $p = $path->child("MYMETA.json"))) {
+    $testee_top_module = JSON->new->decode($p->slurp())->{name};
+    $testee_top_module =~ s/-/::/g;
+    last;
+  }
+}
 oops unless $testee_top_module;
 
 sub verif_no_internals_mentioned($) { # croaks if references found
   my $original = shift;
-  return if $Carp::Verbose; 
+  return if $Carp::Verbose;
 
   local $_ = $original;
 
   # Ignore glob refs like \*{"..."}
   s/(?<!\\)\\\*\{"[^"]*"\}//g;
-  
+
   # Ignore globs like *main::STDOUT or *main::$f
   s/(?<!\\)\*\w[\w:\$]*\b//g;
-  
+
   # Ignore object refs like Some::Package=THING(hexaddr)
   s/(?<!\w)\w[\w:\$]*=(?:REF|ARRAY|HASH|SCALAR|CODE|GLOB)\(0x[0-9a-f]+\)//g;
-  
+
   # Ignore Data::Dumper::addrvis output like Some::Package<dec:hex>
   s/(?<!\w)\w[\w:\$]*<\d+:[\da-f]+>//g;
-  
+
   # Mask references to our test library files named t_something.pm
   s#\b(\bt_\w+).pm(\W|$)#<$1 .pm>$2#gs;
-  
+
   my $msg;
   if (/\b(?<hit>${testee_top_module}::[\w:]*)/) {
     $msg = "ERROR: Log msg or traceback mentions internal package '$+{hit}'"
@@ -310,7 +398,7 @@ sub verif_no_internals_mentioned($) { # croaks if references found
     my $end   = $+[1]; # offset of end+1
     substr($_,$start,0) = "HERE>>>";
     substr($_,$end+7,0) = "<<<THERE";
-    local $Carp::Verbose = 0;  # no full traceback 
+    local $Carp::Verbose = 0;  # no full traceback
     $Carp::CarpLevel++;
     croak $msg, ":\n«$_»\n";
   }
@@ -325,7 +413,7 @@ sub show_white(_) { # show whitespace which might not be noticed
   return "(Is undef)" unless defined;
   s/\t/<tab>/sg;
   s/( +)$/"<space>" x length($1)/seg; # only trailing spaces
-  s/\n/<newline>\n/sg;
+  s/\R/<newline>\n/sg;
   show_empty_string $_
 }
 
@@ -336,7 +424,7 @@ our @quotes = ("«", "»");
 sub rawstr(_) { # just the characters in French Quotes (truncated)
   # Show spaces visibly
   my $text = $_[0];
-  $text =~ s/ /\N{MIDDLE DOT}/gs;
+  ##$text =~ s/ /\N{MIDDLE DOT}/gs;
   $quotes[0].(length($text)>$showstr_maxlen ? substr($text,0,$showstr_maxlen-3)."..." : $text).$quotes[1]
 }
 
@@ -354,26 +442,29 @@ sub showstr(_) {
   if (defined &Data::Dumper::Interp::visnew) {
     return visnew->Useqq("unicode")->vis(shift);
   } else {
-    # I don't want to require Data::Dumper::Interp to be 
+    # I don't want to require Data::Dumper::Interp to be
     # loaded although it will be if t_Common.pm was used also.
     return showcontrols(shift);
   }
 }
 
-# Show both the raw string in French Quotes, and with hex escapes
+# Show the raw string in French Quotes.
+# If STDOUT is not UTF-8 encoded, also show D::D hex escapes
 # so we can still see something useful in output from non-Unicode platforms.
 sub displaystr($) {
   my ($input) = @_;
   return "undef" if ! defined($input);
-  # Data::Dumper will show 'wide' characters as hex escapes
-  my $dd = Data::Dumper->new([$input])->Useqq(1)->Terse(1)->Indent(0)->Dump;
-  chomp $dd;
-  if ($dd eq $input || $dd eq "\"$input\"") {
-    # No special characters, so omit the hex-escaped form
-    return rawstr($input)
-  } else {
-    return rawstr($input)."($dd)"
+  local $_;
+  state $utf8_output = grep /utf.?8/i, PerlIO::get_layers(*STDOUT, output=>1);
+  my $r = rawstr($input);
+  if (! $utf8_output && $input =~ /[^[:print:]]/a) {
+    # Data::Dumper will show 'wide' characters as hex escapes
+    my $dd = Data::Dumper->new([$input])->Useqq(1)->Terse(1)->Indent(0)->Dump;
+    if ($dd ne $input && $dd ne "\"$input\"") {
+      $r .= "\nD::D->$dd";
+    }
   }
+  $r
 }
 
 sub fmt_codestring($;$) { # returns list of lines
@@ -459,12 +550,9 @@ sub expect1($$) {
 # into a regex which works with all versions.
 # As of 1/1/23 the input string is expected to be what Perl v5.34 produces.
 our $bs = '\\';  # a single backslash
-sub expstr2re($) {
+sub _expstr2restr($) {
   local $_ = shift;
   confess "bug" if ref($_);
-  unless (m#qr/|"::#) {
-    return $_; # doesn't contain variable-representation items
-  }
   # In \Q *string* \E the *string* may not end in a backslash because
   # it would be parsed as (\\)(E) instead of (\)(\E).
   # So change them to a unique token and later replace problematic
@@ -479,7 +567,7 @@ sub expstr2re($) {
     # Alternate: qr/(?^MODIFIERS:STUFF)/
     # Alternate: qr/(?^uMODIFIERS:STUFF)/
 #say "#XX qr BEFORE: $_";
-    s#qr/([^\/]+)/([msixpodualngcer]*)
+    s#qr/((?:\\.|[^\/])+)/([msixpodualngcer]*)
      #\\E\(\\Qqr/$1/\\Eu?\\Q$2\\E|\\Qqr/(?^\\Eu?\\Q$2:$1)/\\E\)\\Q#xg
       or confess "Problem with qr/.../ in input string: $_";
 #say "#XX qr AFTER : $_";
@@ -503,10 +591,26 @@ sub expstr2re($) {
   s/<BS>/\\/g;
 #say "#XX    FINAL : $_";
 
-  my $saved_dollarat = $@;
-  my $re = eval "qr{${_}}"; die "$@ " if $@;
-  $@ = $saved_dollarat;
-  $re
+  $_
+}
+sub expstr2re($) {
+  my $input = shift;
+  my $xdesc; # extra debug description of intermediates
+  my $output;
+  if ($input !~ m#qr/|"::#) {
+    # doesn't contain variable-representation items
+    $output = $input;
+    $xdesc = "";
+  } else {
+    my $s = _expstr2restr($input);
+    my $saved_dollarat = $@;
+    my $re = eval "qr{$s}"; die "$@ " if $@;
+    $@ = $saved_dollarat;
+    $xdesc = "**Orig match str  :".displaystr($input)."\n"
+            ."**Generated re str:".displaystr($s)."\n" ;
+    $output = $re;
+  }
+  wantarray ? ($xdesc, $output) : $output
 }
 
 # check $test_desc, string_or_regex, result
@@ -530,15 +634,18 @@ sub mycheck($$@) {
   foreach my $i (0..$#actual) {
     my $actual = $actual[$i];
     my $expected = $expected[$i];
+    my $xdesc = "";
     if (!ref($expected)) {
       # Work around different Perl versions stringifying regexes differently
-      $expected = expstr2re($expected);
+      #$expected = expstr2re($expected);
+      ($xdesc, $expected) = expstr2re($expected);
     }
     if (ref($expected) eq "Regexp") {
       unless ($actual =~ $expected) {
         @_ = ( "\n**************************************\n"
               ."TESTb FAILED: ".$desc."\n"
               ."Expected (Regexp):\n".${expected}."<<end>>\n"
+              .$xdesc
               ."Got:\n".displaystr($actual)."<<end>>\n"
              ) ;
         Carp::confess(@_); #goto &Carp::confess;
@@ -561,17 +668,14 @@ sub verif_eval_err(;$) {  # MUST be called on same line as the 'eval'
   my $fn = $caller[1];
   my $ex = $@;
   confess "expected error did not occur at $fn line $ln\n",
-          fmtsheet(sheet({package => $caller[0]}))
     unless $ex;
 
-  if ($ex !~ / at $fn line $ln\.?(?:$|\n)/s) {
+  if ($ex !~ / at \Q$fn\E line $ln\.?(?:$|\R)/s) {
     confess "Got UN-expected err (not ' at $fn line $ln'):\n«$ex»\n",
-            fmtsheet(sheet({package => $caller[0]})),
             "\n";
   }
   if ($msg_regex && $ex !~ qr/$msg_regex/) {
     confess "Got UN-expected err (not matching $msg_regex) at $fn line $ln'):\n«$ex»\n",
-            fmtsheet(sheet({package => $caller[0]})),
             "\n";
   }
   verif_no_internals_mentioned($ex);
@@ -588,7 +692,7 @@ sub insert_loc_in_evalstr($) {
 sub timed_run(&$@) {
   my ($code, $maxcpusecs, @codeargs) = @_;
 
-  my $getcpu = eval {do{ 
+  my $getcpu = eval {do{
     require Time::HiRes;
     () = (&Time::HiRes::clock());
     \&Time::HiRes::clock;
